<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class TipoDeCardapioController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
  protected $connection;
  protected $utilities;
  protected $errorCode = [
		"ORA-02292" => "ERROR_REGISTROFILHO"
	];

    public function __construct($crudService, $parameterBag, $connection, $environment, $utilities) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
        $this->utilities = $utilities;
	}
	
	public function saveArvTpCard(DTO\Request $request, DTO\Response $response) {
	    try {

          $row = $request->getRow();

          $this->connection->beginTransaction();

          $sequencial = $this->getSeqArvCard($row) + 1;
          $sequencial = str_pad($sequencial, 10, "0", STR_PAD_LEFT);
          $nrNivel = empty($row['NIVELPAI']) ? 1 : $row['NIVELPAI'] + 1;

          $query = "INSERT INTO ARVTIPOCARD (CDFILCENTPME, NRPASTATCARD, NMPASTATCARD, NRNVPASTCARD, NRPASTCARANT, DTULTATUARVT)
                    VALUES (:CDFILCENTPME, :NRPASTATCARD, :NMPASTATCARD, :NRNVPASTCARD, :NRPASTCARANT, SYSDATE)";
            
          $params = array(
            'CDFILCENTPME' => $row['CDFILCENTPME'],
            'NRPASTATCARD' => $sequencial,
            'NMPASTATCARD' => $row['NMPASTATCARD'],
            'NRNVPASTCARD' => $nrNivel,
            'NRPASTCARANT' => $row['NRPASTAPAI']
          );
            
	        $this->connection->prepare($query)->execute($params);
	        
	        $this->connection->commit();
	        $response->addDataSet(new DataSet("save", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  public function saveTipoCard(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = $request->getRow();
      $rowsToSave = [];
      
      foreach($rows as $row){
        $row['CDTIPOCARD'] = str_pad($row['CDTIPOCARD'], 10, "0", STR_PAD_LEFT);
        $row['DTULTATUTCAR'] = date('d/m/Y');
        $row['DSPASTATCARD'] = null;
		    array_push($rowsToSave, $row);
		  }
      $dataSourceName = 'tipocard';
      $this->callSave($dataSourceName, $rowsToSave);
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveServTpCard(DTO\Request $request, DTO\Response $response) {
    try {

        $this->connection->beginTransaction();

        $rows = (array) $request->getRow();
        $dataSourceName = 'servTipoCard';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveCusto(DTO\Request $request, DTO\Response $response) {
    try {

        $this->connection->beginTransaction();

        $rows = $request->getRow();
        $rowsToSave = [];

        foreach($rows as $row){
          array_push($rowsToSave, $row);
        }
        $dataSourceName = 'custoPadrCard';
        $this->callSave($dataSourceName, $rowsToSave);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveConfigPrat(DTO\Request $request, DTO\Response $response) {
    $this->connection->beginTransaction();
    try {
      $rows = (Array) $request->getRow();
      // $this->deleteConfigPrat($rows);
      $errors = [];
      $nrMaxSeq = $this->getMaxSeqConfPrat($rows[0])['NRSEQCNFTCAR'];
      $newRows = [];
      foreach ($rows as $row) {
        $row['__is_new'] = true;
        if ($row['NEW'] === 'S') {
          // $validacao = $this->validConfigPrat($row);
          $validacao = true;
          if ($validacao !== true) {
            $errors[] = $validacao;
            continue;
          } else {
            $nrMaxSeq += 1;
            $row['NRSEQCNFTCAR'] = $nrMaxSeq;
            $row['NRORDCNFTCAR'] = $nrMaxSeq;
            $newRows[] = $row;
          }
        } else $newRows[] = $row;
      }
      $rows = $newRows;
      $this->deleteConfigPrat($rows);
      $dataSourceName = 'confpadtcard';
      $this->callSave($dataSourceName, $rows);
      $this->connection->commit();
      if (!empty($errors)) {
        $returnError = 'impAdicionarExistentes: '.implode(", ", $errors);
        $response->addDataset(new DataSet('mensagem', array($returnError)));
        return;
      }
      // $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  private function validConfigPrat($row) {
    $nrorg = $this->environment->getNrOrg();
    
    $query = "SELECT CF.CDTIPOPRAT, TC.NMTIPOPRAT FROM CONFPADTCARD CF
            JOIN TIPOPRAT TC ON CF.CDTIPOPRAT = TC.CDTIPOPRAT
           WHERE CF.CDFILCENTPME = :CDFILCENTPME
             AND CF.CDSERVFILI = :CDSERVFILI
             AND CF.CDTIPOPRAT = :CDTIPOPRAT
             AND CF.CDTIPOCARD = :CDTIPOCARD";
  
    $params = array(
      'CDFILCENTPME' => $row['CDFILCENTPME'],
      'CDSERVFILI' => $row['CDSERVFILI'],
      'CDTIPOPRAT' => $row['CDTIPOPRAT'],
      'CDTIPOCARD' => $row['CDTIPOCARD']
    );
    $result = $this->connection->fetchAll($query, $params);
    if (count($result) > 0) return $result[0]['NMTIPOPRAT'];
    return true;
  }
  
  private function deleteConfigPrat($rows) {
    $nrorg = $this->environment->getNrOrg();
    
      foreach($rows as $row) {
        $query = "DELETE
                FROM CONFPADTCARD
               WHERE CDFILCENTPME = :CDFILCENTPME
                 AND CDSERVFILI = :CDSERVFILI
                 AND CDTIPOPRAT = :CDTIPOPRAT
                 AND NRSEQCNFTCAR = :NRSEQCNFTCAR";
      
        $params = array(
          'CDFILCENTPME' => $row['CDFILCENTPME'],
          'CDSERVFILI' => $row['CDSERVFILI'],
          'CDTIPOPRAT' => $row['CDTIPOPRAT'],
          'NRSEQCNFTCAR' => $row['NRSEQCNFTCAR']
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
      }
  }

  
  private function getMaxSeqConfPrat($row) {
    $sql = "SELECT MAX(NRSEQCNFTCAR) NRSEQCNFTCAR
                  FROM CONFPADTCARD
                  WHERE CDFILCENTPME = :CDFILCENTPME
                        AND CDTIPOCARD = :CDTIPOCARD
                        AND CDSERVFILI = :CDSERVFILI";
    $params = array(
      "CDFILCENTPME" => $row['CDFILCENTPME'],
      "CDTIPOCARD" => $row['CDTIPOCARD'],
      "CDSERVFILI" => $row['CDSERVFILI'],
    );
    $resultado = $this->connection->fetchAssoc($sql, $params);
    return $resultado;
  }

  public function saveUnidadeTipoCard(DTO\Request $request, DTO\Response $response) {
    try {
        $this->connection->beginTransaction();
        $rows = (Array) $request->getRow();
        $unidadesSemParamPlaneja = [];
        foreach ($rows as $idx => $row) {
          $hasParamPlaneja = $this->utilities->checkParamPlaneja($row['CDFILIAL']);
          if ($hasParamPlaneja !== true) {
            $unidadesSemParamPlaneja[] = $hasParamPlaneja;
				    unset($rows[$idx]);
          }
        }
        $dataSourceName = 'filitipocardUnidade';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        if (!empty($unidadesSemParamPlaneja)) {
          $response->addDataset(new DataSet('mensagem', array('ERROR_AGREGADA_SEM_PARAMPLANEJA')));
			    $response->addDataset(new DataSet('comp', array_unique($unidadesSemParamPlaneja)));
        }
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function saveTipoCardUnidade(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $rows = (Array) $request->getRow();
      $unidadesSemParamPlaneja = [];
      foreach ($rows as $idx => $row) {
        $hasParamPlaneja = $this->utilities->checkParamPlaneja($row['CDFILIAL']);
        if ($hasParamPlaneja !== true) {
          $unidadesSemParamPlaneja[] = $hasParamPlaneja;
			    unset($rows[$idx]);
        }
      }
      $dataSourceName = 'filitipocardTipoCard';
      $this->callSave($dataSourceName, $rows);
      $this->connection->commit();
      if (!empty($unidadesSemParamPlaneja)) {
        $response->addDataset(new DataSet('mensagem', array('ERROR_AGREGADA_SEM_PARAMPLANEJA')));
		    $response->addDataset(new DataSet('comp', array_unique($unidadesSemParamPlaneja)));
      }
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function callSave($dataSourceName, $rowsToSave){
    $dataSet = new DataSet($dataSourceName, $rowsToSave);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);    
    $this->crudService->save($dataSourceName, $rowsToSave);
  }
	
	private function getSeqArvCard($row) {
    $query = "SELECT MAX(NRPASTATCARD) MAXNRPASTATCARD FROM ARVTIPOCARD WHERE CDFILCENTPME =:CDFILCENTPME";
    $params = array(
      'CDFILCENTPME' => $row['CDFILCENTPME']
    );
		return $this->connection->fetchColumn($query, $params);
  }
  
  public function deleteTree(DTO\Request $request, DTO\Response $response) {
    try {
        $nrorg = $this->environment->getNrOrg();

        $this->connection->beginTransaction();

        $rows = $request->getRow();
        foreach($rows as $row) {
          $query = "DELETE FROM ARVTIPOCARD
                 WHERE CDFILCENTPME = :CDFILCENTPME
                   AND NRPASTATCARD = :NRPASTATCARD";
        
          $params = array(
            'CDFILCENTPME' => $row['CDFILCENTPME'],
            'NRPASTATCARD' => $row['NRPASTATCARD']
          );
          $stmt = $this->connection->prepare($query)->execute($params);
        }
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", []));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $errorMessage = $e->getMessage();
			foreach($this->errorCode as $key => $val){
				if(strpos($errorMessage, $key)) $errorMessage = $val;
			}
			$response->setError(new Error($errorMessage, 500));
    }
  }
  
  public function importaConfigPadraoTpCard(DTO\Request $request, DTO\Response $response) {
    try {
        $parameters = $request->getParameters();
        $rows = $parameters['rowsGridConf'];
        $paramsInsert = $parameters['parametrosDaQueryInsert'];
        
        $this->connection->beginTransaction();


        if ($rows) {
          $query = "DELETE
                    FROM CONFPADTCARD
                   WHERE CDFILCENTPME = :OLD_CDFILCENTPME
                     AND CDTIPOCARD = :OLD_CDTIPOCARD
                     AND CDSERVFILI = :OLD_CDSERVFILI";
            
          foreach($rows as $row){
            $params = array(
              'OLD_CDFILCENTPME' => $row['CDFILCENTPME'],
              'OLD_CDTIPOCARD' => $row['CDTIPOCARD'],
              'OLD_CDSERVFILI' => $row['CDSERVFILI'],
            );
            $this->connection->prepare($query)->execute($params);
          }
        }
        
        $query = "INSERT INTO CONFPADTCARD (CDFILCENTPME, CDTIPOCARD, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR)
                (SELECT CDFILIAL, :P_CDTIPOCARD, CDSERVFILI, NRSEQCNFCARD, CDTIPOPRAT, IDOPCCNFCARD, IDIMPCNFCARD, NRORDCNFCARD
                   FROM CONFCARD
                  WHERE (CDFILIAL = :P_CDFILIAL) 
                    AND (CDSERVFILI = :P_CDSERVFILI)
                )";
            
        $params = array(
          'P_CDTIPOCARD' => $paramsInsert['CDTIPOCARD'],
          'P_CDFILIAL' => $paramsInsert['CDFILIAL'],
          'P_CDSERVFILI' => $paramsInsert['CDSERVFILI'],
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        
        $this->connection->commit();
        $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
    	$this->connection->rollback();
		  $response->setError(new Error($e->getMessage(), 500));
	  }
  }
  
	public function getDataConfigPadraoTpCard(DTO\Request\Row $request, DTO\Response $response) {
		$sql = "SELECT CDFILIAL, CDSERVFILI, NRORG FROM CONFCARD WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI";
		$row = $request->getRow();
		$paramSelect = array(
        "CDSERVFILI" => $row['CDSERVFILI'],
        "CDFILIAL"	 => $row['CDFILIAL']
    );
      
    $dataConfigPadrao = $this->connection->fetchAll($sql, $paramSelect);
    $response->addDataSet(new DataSet("result", $dataConfigPadrao));
	}
	
	public function updateTree(DTO\Request\Row $request, DTO\Response $response) {
	  try {
      $parameters = $request->getParameters();
      $rows = $parameters['rowsEditadas'];
      $nrorg = $this->environment->getNrOrg();
      
      $this->connection->beginTransaction();

      $query = "UPDATE ARVTIPOCARD 
                  SET 
                    NMPASTATCARD = :P_NMPASTATCARD
                  WHERE NRPASTATCARD = :P_NRPASTATCARD
                    AND CDFILCENTPME = :P_CDFILCENTPME
                    AND NRORG        = :P_NRORG";
        
      foreach($rows as $row){
        $params = array(
          'P_NMPASTATCARD' => $row['NMPASTATCARD'],
          'P_NRPASTATCARD' => $row['NRPASTATCARD'],
          'P_CDFILCENTPME' => $row['CDFILCENTPME'],
          'P_NRORG' => $nrorg
        );
        $this->connection->prepare($query)->execute($params);
      }
          
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
	  } catch (\Exception $e) {
	    $this->connection->rollback();
	    $response->setError(new Error($e->getMessage(), 500));
	  }
	}
	
	public function updateTipoCard(DTO\Request $request, DTO\Response $response) {
    try {
      $parameters = $request->getParameters();
      $rows = $parameters['rows'];
      $nrorg = $this->environment->getNrOrg();

      $this->connection->beginTransaction();

      $query = "UPDATE TIPOCARD 
                SET 
                  CDGRUPTIPOCARD = :P_CDGRUPTIPOCARD
                WHERE CDTIPOCARD   = :P_CDTIPOCARD
                  AND CDFILCENTPME = :P_CDFILCENTPME
                  AND NRORG        = :P_NRORG";

      foreach($rows as $row) {
        foreach($row['CDTIPOCARD'] as $cdtipocard) {
          if ($row['ACAO'] === 'insert') $cdAgrupTipoCard = $row['CDGRUPTIPOCARD'];
          else $cdAgrupTipoCard = NULL;
          $params = array(
            'P_CDGRUPTIPOCARD' => $cdAgrupTipoCard,
            'P_CDFILCENTPME' => $row['CDFILCENTPME'],
            'P_CDTIPOCARD'   => $cdtipocard,
            'P_NRORG'        => $nrorg,
          );

          $stmt = $this->connection->prepare($query);
          $stmt->execute($params);
        }
      }

      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
 public function deleteUnidTpCard(DTO\Request $request, DTO\Response $response) {
    try {
      $parameters = $request->getParameters();
      $cardapios = $parameters['checkedRows'];
      
      $this->connection->beginTransaction();

      $deleteQuery = "DELETE FROM FILITIPOCARD 
                      WHERE CDTIPOCARD = :CDTIPOCARD 
                      AND CDFILCENTPME = :CDFILCENTPME 
                      AND CDFILIAL = :CDFILIAL";

      $totalDeleted = 0; // Contador de registros excluídos
      $totalFailed = 0; // Contador de falhas

      foreach ($cardapios as $row) {
        if (isset($row['GRID']) && $row['GRID'] === 'UNIDADECARD' && isset($row['CDFILIAL']) && is_array($row['CDFILIAL'])) {
          // excluir por unidade de cardápio
          foreach ($row['CDFILIAL'] as $cdFilial) {
            $params = array(
              'CDTIPOCARD'   => $row['CDTIPOCARD'],
              'CDFILCENTPME' => $row['CDFILCENTPME'],
              'CDFILIAL'     => $cdFilial
            );
            
            try {
              $this->connection->executeQuery($deleteQuery, $params);
              $totalDeleted++;
            } catch (\Exception $e) {
              $totalFailed++;
            }
          }
        } elseif (isset($row['CDTIPOCARD']) && is_array($row['CDTIPOCARD'])) {
          // excluir por tipo de cardápio
          foreach ($row['CDTIPOCARD'] as $cdTipoCard) {
            $params = array(
              'CDTIPOCARD'   => $cdTipoCard,
              'CDFILCENTPME' => $row['CDFILCENTPME'],
              'CDFILIAL'     => $row['CDFILIAL']
            );
            
            try {
              $this->connection->executeQuery($deleteQuery, $params);
              $totalDeleted++;
            } catch (\Exception $e) {
              $totalFailed++;
            }
          }
        }
      }

      $this->connection->commit();

      $response->addDataSet(new DataSet("success", array(true)));
      $response->addDataSet(new DataSet("totalDeleted", array($totalDeleted)));
      $response->addDataSet(new DataSet("totalFailed", array($totalFailed)));

    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}